
#include "stdafx.h"
#include "video.h"
#include "VideoShowCtl.h"

#include <vfw.h>

void bmphprint(BITMAPINFOHEADER* pb);

IMPLEMENT_DYNCREATE(CVideoShowCtrl, COleControl)

BEGIN_MESSAGE_MAP(CVideoShowCtrl, COleControl)
	//{{AFX_MSG_MAP(CVideoShowCtrl)
	//}}AFX_MSG_MAP
	ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CVideoShowCtrl, COleControl)
	//{{AFX_DISPATCH_MAP(CVideoShowCtrl)
	DISP_PROPERTY(CVideoShowCtrl, "AutoSize", m_autoSize, VT_BOOL)
	DISP_PROPERTY(CVideoShowCtrl, "Stretch", m_stretch, VT_BOOL)
	DISP_PROPERTY_NOTIFY(CVideoShowCtrl, "AutoRedraw", m_autoRedraw, OnAutoRedrawChanged, VT_BOOL)
	DISP_FUNCTION(CVideoShowCtrl, "Draw", Draw, VT_EMPTY, VTS_I4 VTS_I4)
	DISP_PROPERTY_PARAM(CVideoShowCtrl, "Picture", GetPicture, SetNotSupported, VT_PICTURE, VTS_I4 VTS_I4)
	DISP_STOCKPROP_BACKCOLOR()
	//}}AFX_DISPATCH_MAP
	DISP_FUNCTION_ID(CVideoShowCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()

BEGIN_EVENT_MAP(CVideoShowCtrl, COleControl)
	//{{AFX_EVENT_MAP(CVideoShowCtrl)
	EVENT_CUSTOM("Resize", FireResize, VTS_NONE)
	//}}AFX_EVENT_MAP
END_EVENT_MAP()

IMPLEMENT_OLECREATE_EX(CVideoShowCtrl, "VIDEO.VideoShowCtrl.1",	0x96872fa7, 0xd44c, 0x11d1, 0xa8, 0x98, 0, 0x20, 0xaf, 0xf8, 0x4c, 0xb3)

IMPLEMENT_OLETYPELIB(CVideoShowCtrl, _tlid, _wVerMajor, _wVerMinor)

const IID BASED_CODE IID_DVideoShow = { 0x96872fa5, 0xd44c, 0x11d1, { 0xa8, 0x98, 0, 0x20, 0xaf, 0xf8, 0x4c, 0xb3 } };
const IID BASED_CODE IID_DVideoShowEvents = { 0x96872fa6, 0xd44c, 0x11d1, { 0xa8, 0x98, 0, 0x20, 0xaf, 0xf8, 0x4c, 0xb3 } };

static const DWORD BASED_CODE _dwVideoShowOleMisc = OLEMISC_ACTIVATEWHENVISIBLE | OLEMISC_SETCLIENTSITEFIRST | OLEMISC_INSIDEOUT | OLEMISC_CANTLINKINSIDE | OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CVideoShowCtrl, IDS_VIDEOSHOW, _dwVideoShowOleMisc)

BOOL CVideoShowCtrl::CVideoShowCtrlFactory::UpdateRegistry(BOOL bRegister)
{
	if (bRegister)
		return AfxOleRegisterControlClass(
			AfxGetInstanceHandle(),
			m_clsid,
			m_lpszProgID,
			IDS_VIDEOSHOW,
			IDB_VIDEOSHOW,
			afxRegApartmentThreading,
			_dwVideoShowOleMisc,
			_tlid,
			_wVerMajor,
			_wVerMinor);
	else
		return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}

CVideoShowCtrl::CVideoShowCtrl()
{
	InitializeIIDs(&IID_DVideoShow, &IID_DVideoShowEvents);
	OnResetState();
   hdcMem=0; hbitmap=0;
}

CVideoShowCtrl::~CVideoShowCtrl()
{
   if (hdcMem) DeleteDC(hdcMem);
   if (hbitmap) DeleteObject(hbitmap);
}

void CVideoShowCtrl::OnDraw(CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
   if (!AmbientUserMode()) { DrawOcxBitmap(this, pdc, rcBounds, IDB_VIDEOSHOW); return; }
   CBrush cb; cb.CreateSolidBrush(TranslateColor(GetBackColor())); pdc->FillRect(rcBounds, &cb);

   if (hdcMem && m_autoRedraw) 
   {
      int x, y; 
      if (m_stretch) GetControlSize(&x, &y); else { x=m_width; y=m_height; }
      StretchBlt(pdc->m_hDC, 0, 0, x, y, hdcMem, 0, 0, m_width, m_height, SRCCOPY);
   }
}

void CVideoShowCtrl::DoPropExchange(CPropExchange* pPX)
{
   ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
	COleControl::DoPropExchange(pPX);

   PX_Bool(pPX, "AutoSize", m_autoSize, TRUE);
   PX_Bool(pPX, "AutoRedraw", m_autoRedraw, TRUE);
}

void CVideoShowCtrl::OnResetState()
{
   COleControl::OnResetState();  // Resets defaults found in DoPropExchange
   m_width=-1;
   m_height=-1;
}

void CVideoShowCtrl::AboutBox()
{
	CDialog dlgAbout(IDD_ABOUTBOX);
	dlgAbout.DoModal();
}

// ******************************************************************

void CVideoShowCtrl::Draw(long bmph, long data) 
{
   BITMAPINFOHEADER *b=(BITMAPINFOHEADER*)bmph;
   if (!data) return;
   CDC *dc=GetDC();

   if (m_width!=b->biWidth || m_height!=b->biHeight)
   {
      m_width=b->biWidth; m_height=b->biHeight;
      if (m_autoSize) { SetControlSize(b->biWidth, b->biHeight); FireResize(); }

      if (m_autoRedraw)
      {
         if (hdcMem) DeleteDC(hdcMem);
         if (hbitmap) DeleteObject(hbitmap);
         hdcMem=CreateCompatibleDC(dc->m_hDC);
         hbitmap=CreateCompatibleBitmap(dc->m_hDC, m_width, m_height);
         SelectObject(hdcMem, hbitmap);
      }
   }

   int x, y; 
   if (m_stretch) GetControlSize(&x, &y); else { x=b->biWidth; y=b->biHeight; }

   HDRAWDIB hdd=DrawDibOpen();
   if (m_autoRedraw)
   {
      DrawDibDraw(hdd, hdcMem, 0, 0, b->biWidth, b->biHeight, b, (void*)data, 0, 0, b->biWidth, b->biHeight, 0); //DDF_SAME_DRAW|DDF_SAME_HDC); 
      StretchBlt(dc->m_hDC, 0, 0, x, y, hdcMem, 0, 0, m_width, m_height, SRCCOPY);
   }
   else
      DrawDibDraw(hdd, dc->m_hDC, 0, 0, x, y, b, (void*)data, 0, 0, b->biWidth, b->biHeight, 0); //DDF_SAME_DRAW|DDF_SAME_HDC); 
   DrawDibClose(hdd);

   ReleaseDC(dc);
}

/*
void CVideoShowCtrl::Save(LPCTSTR filename, long bmph, long data) 
{
   BITMAPINFOHEADER *b=(BITMAPINFOHEADER*)bmph;
   BITMAPFILEHEADER hdr;
   unsigned long imagesize=b->biSizeImage?b->biSizeImage:b->biWidth*b->biHeight*b->biBitCount/8;

   FILE* fp=fopen((char*)"c:\\test.bmp", "w");
   hdr.bfType      = ((WORD) ('M' << 8) | 'B');    // is always "BM"
   hdr.bfSize      = sizeof(hdr) + b->biSize + imagesize;
   hdr.bfReserved1 = 0;
   hdr.bfReserved2 = 0;
   hdr.bfOffBits   = (DWORD) (sizeof(hdr) + b->biSize); //+ (1 << b->biBitCount) * sizeof(RGBQUAD));
   fwrite((char*)&hdr, 1, sizeof(hdr), fp);
   fwrite((char*)bmph, 1, b->biSize, fp);
   fwrite((char*)data, 1, imagesize, fp);
   fwrite((char*)filename, 1, strlen((char*)filename), fp);
   fclose(fp);
*

   BITMAPINFOHEADER *bh=(BITMAPINFOHEADER *)bmph;
   HANDLE hDIB=CreateBitmap(bh->biWidth, bh->biHeight, bh->biPlanes, bh->biBitCount, (void*)data);

   // http://www.codeguru.com/bitmap/bitmap_to_file.shtml
   BITMAPFILEHEADER hdr;
   LPBITMAPINFOHEADER lpbi;
   if (!hDIB) return;
   CFile file;
   if (!file.Open("c:\\daten\\tst2.bmp"/*filename, CFile::modeWrite|CFile::modeCreate)) return;
   lpbi = (LPBITMAPINFOHEADER)hDIB;
   int nColors = 1 << lpbi->biBitCount;
   hdr.bfType      = ((WORD) ('M' << 8) | 'B');    // is always "BM"
   hdr.bfSize      = GlobalSize (hDIB) + sizeof( hdr );
   hdr.bfReserved1 = 0;
   hdr.bfReserved2 = 0;
   hdr.bfOffBits   = (DWORD) (sizeof( hdr ) + lpbi->biSize + nColors * sizeof(RGBQUAD));
   file.Write(&hdr, sizeof(hdr) );      // Write the file header 
   file.Write(lpbi, GlobalSize(hDIB) ); // Write the DIB header and the bits 
   file.Close();
*
}
*/

void CVideoShowCtrl::OnAutoRedrawChanged() 
{
	if (!m_autoRedraw)
   {
      if (hdcMem) DeleteDC(hdcMem);
      if (hbitmap) DeleteObject(hbitmap);
      hdcMem=0; hbitmap=0;
   }

	SetModifiedFlag();
}

LPPICTUREDISP CVideoShowCtrl::GetPicture(long bmph, long data)
{
   CPictureHolder picHolder;

   /*
   BITMAPINFOHEADER *bm=(BITMAPINFOHEADER*)b;
   BITMAPFILEHEADER hdr;
   unsigned long imagesize=bm->biSizeImage?bm->biSizeImage:bm->biWidth*bm->biHeight*bm->biBitCount/8;
   FILE* fp=fopen((char*)"c:\\daten\\xxx2.bmp", "w");
   hdr.bfType      = ((WORD) ('M' << 8) | 'B');    // is always "BM"
   hdr.bfSize      = sizeof(hdr) + capGetVideoFormat(hWnd,0,0)+ imagesize;
   hdr.bfReserved1 = 0;
   hdr.bfReserved2 = 0;
   hdr.bfOffBits   = (DWORD) (sizeof(hdr) +capGetVideoFormat(hWnd,0,0)); //+ (1 << b->biBitCount) * sizeof(RGBQUAD));
   fwrite((char*)&hdr, 1, sizeof(hdr), fp);
   fwrite((char*)bm, 1, capGetVideoFormat(hWnd,0,0), fp);
   fwrite((char*)data, 1, imagesize, fp);
   fclose(fp);
   */

   BITMAPINFO *bh=(BITMAPINFO *)bmph;
   PICTDESC pPictDesc;
   pPictDesc.cbSizeofstruct=sizeof(PICTDESC);
   pPictDesc.picType=PICTYPE_BITMAP;
   pPictDesc.bmp.hbitmap=CreateDIBitmap(GetDC()->m_hDC, &(bh->bmiHeader), CBM_INIT, (void*)data, bh, DIB_RGB_COLORS);
   // LOGPALETTE pal;
   // pal.palVersion=0x300;
   // pal.palNumEntries=(unsigned short)bh->biClrUsed;
   // PALETTEENTRY *bmiColors=(PALETTEENTRY *)((char*)bh+bh->biSize);
   // for(unsigned int i=0;i<bh->biClrUsed;i++) pal.palPalEntry[i]=bmiColors[i];
   // pPictDesc.bmp.hpal=CreatePalette(&pal);
   pPictDesc.bmp.hpal=0;
   OleCreatePictureIndirect(&pPictDesc, IID_IPicture, FALSE, (void**)&picHolder.m_pPict);

	return picHolder.GetPictureDispatch();
}
